"""Import m3u files."""

import re
from dataclasses import dataclass, field
from .subgroup import subgroup_vod_only
from enum import Enum


class ChannelType(Enum):
    """Channeltypes for M3uChannel."""

    UNKNOWN = 0
    CHANNEL = 1
    EPISODE = 2
    MOVIE = 3


@dataclass
class M3uChannel:
    """Class for M3uChannel."""

    tvg_id: str
    tvg_name: str
    tvg_logo: str
    tvg_group_title: str
    vod: bool = False
    tvg_sources: list = field(default_factory=list)
    tvg_type: ChannelType = ChannelType.UNKNOWN
    fhs_selected: bool = False
    fhs_tags: set = field(default_factory=set)
    fhs_dict: dict = field(default_factory=dict)


def import_m3u_stream(input_str):
    """Import m3u stream.

    Args:
        input_str: file handler to read

    Returns:
        list of m3uchannel
    """
    m3uchannels = []
    # regex found on iptv-filter
    infopattern = re.compile('(?i)#EXTINF:-1 tvg-id="(.*?)" tvg-name="(.*?)" tvg-logo="(.*?)" group-title="(.*?)",(.*?)')  # noqa: E501,BLK100
    urlpattern = re.compile('(?i)^http')
    for line in input_str:
        clean_line = line.rstrip()
        m = infopattern.findall(clean_line)
        if len(m) > 0:
            # #EXTINF line
            tvg_id = m[0][0]
            tvg_name = m[0][1]
            tvg_logo = m[0][2]
            tvg_group_title = m[0][3]
            m3uchannels.append(M3uChannel(tvg_id=tvg_id, tvg_name=tvg_name.strip(), tvg_logo=tvg_logo, tvg_group_title=tvg_group_title.strip()))  # noqa: E501
        else:
            if urlpattern.match(clean_line):
                # This is the URL Line
                m3uchannels[-1].tvg_sources.append(clean_line)
                if clean_line.endswith(('.mkv', '.avi', '.mp4')) is True:
                    m3uchannels[-1].vod = True
    if len(m3uchannels) == 0:
        return None
    return m3uchannels


def import_m3u_file(m3u_file):
    """Import m3u_file.

    Args:
        m3u_file: path to m3u file with channels/vod

    Returns:
        returns list op channnels/vods
    """
    try:
        with open(m3u_file, 'r') as input_stream:
            return (import_m3u_stream(input_stream))
    except FileNotFoundError:
        print(f"file {m3u_file} not found.")
    return None


def export_m3u_file(m3u_file, m3u_channels):
    """Export to m3u file from channels.

    Args:
        m3u_file: path to m3u file to write
        m3u_channels: channels to save to file.

    Returns:
        Result
    """
    try:
        with open(m3u_file, "w") as output_stream:
            print("#EXTM3U", file=output_stream)
            for ch in m3u_channels:
                print(f'#EXTINF:-1 tvg-id="{ch.tvg_id}" tvg-name="{ch.tvg_name}" tvg-logo="{ch.tvg_logo}" group-title="{ch.tvg_group_title}",{ch.tvg_name}', file=output_stream)  # noqa: E501
                for source in ch.tvg_sources:
                    print(source, file=output_stream)
    except PermissionError:
        print('ERROR: no permission.')
        return False
    except OSError as e:
        print(f"ERROR: oserror {e}")
        return False
    return True


def return_tvg_group_titles(m3u_channels, vod_only=False):
    """Extract group titles from m3u_channgels.

    Args:
        m3u_channels: list of M3uChanngels dataclasses
        vod_only: boolean of only wants to return vod rules

    Returns:
        list of group titles
    """
    tvg_groups = set()
    if vod_only is True:
        channels = subgroup_vod_only(m3u_channels)
    else:
        channels = m3u_channels
    for ch in channels:
        tvg_groups.add(ch.tvg_group_title)
    return sorted(tvg_groups)


def serie_regex():
    """Get compiled serie regex.

    Returns:
        Compiled regex
    """
    return re.compile('(.*?) S(\d+) {0,1}E(\d+)')  # noqa: W291,W605


def check_type_channel(m3u_channel, compiled_regex=None):
    """Check the channel type.

    Args:
        m3u_channel: Channel M3uChannel dataclass
        compiled_regex: optional re.compile regex

    Returns:
        ChannelType value

    """
    if compiled_regex is None:
        compiled_regex = serie_regex()

    if m3u_channel.vod is False:
        return ChannelType.CHANNEL
    if compiled_regex.findall(m3u_channel.tvg_name):
        return ChannelType.EPISODE
    return ChannelType.MOVIE


def get_channel_types(m3u_channels):
    """Check types of channels.

    Args:
        m3u_channels: List of M3uChannel dataclass

    Returns:
        list of m3u_channels
    """
    compiled_regex = serie_regex()
    result = []
    for i in m3u_channels:
        if i.tvg_sources == []:
            continue
        value = check_type_channel(i, compiled_regex)
        i.tvg_type = value
        # print(f"{i.tvg_name}: {i.tvg_sources[0]} / {i.tvg_type}")
        result.append(i)
    return result


def return_tvg_group_details(m3u_channels, vod_only=False):
    """Extract group titles from m3u_channgels.

    Args:
        m3u_channels: list of M3uChanngels dataclasses
        vod_only: boolean of only wants to return vod rules

    Returns:
        list of group titles
    """
    tvg_groups = dict()
    if vod_only is True:
        channels = subgroup_vod_only(m3u_channels)
    else:
        channels = m3u_channels
    for ch in get_channel_types(channels):
        group_info = tvg_groups.get(ch.tvg_group_title, {'channels': 0, 'episodes': 0, 'movies': 0})
        if ch.tvg_type == ChannelType.CHANNEL:
            group_info['channels'] += 1
        if ch.tvg_type == ChannelType.EPISODE:
            group_info['episodes'] += 1
        if ch.tvg_type == ChannelType.MOVIE:
            group_info['movies'] += 1
        tvg_groups[ch.tvg_group_title] = group_info
    return tvg_groups



"""File for multipe utils."""


def check_search_filter_in_string(full_string, search_string):
    """Check if search filter in sttring.

    Todo: make a regex posible

    Args:
        full_string: string to search in
        search_string: (sub)string to search for

    Returns:
        match: boolean
    """
    if search_string == full_string:
        return True

    if search_string in full_string:
        return True
    return False



"""Get config directories."""

import os
import appdirs
from .__version__ import __app_name__, __author_name__


def get_config_dir():
    """Get config directory.

    Returns:
        config_dir
    """
    config_dir = appdirs.user_config_dir(__app_name__, __author_name__)
    return config_dir


def get_data_dir():
    """Get data directory.

    Returns:
        data_dir
    """
    data_dir = appdirs.user_data_dir(__app_name__, __author_name__)
    return data_dir


def get_probe_dir():
    """Get probe dir.

    Returns:
        probe_dir
    """
    data_dir = get_data_dir()
    probe_dir = os.path.join(data_dir, "ffprobe_data")
    os.makedirs(probe_dir, mode=0o750, exist_ok=True)
    return probe_dir






"""Console script for fhs_iptv_tools."""
import sys

import typer
from .probe import ProbeInfo
from pprint import pprint
from typing import Optional, List, Tuple

from fhs_iptv_tools import config  # noqa: F401

main = typer.Typer()


@main.command()
def probe(source: str = typer.Argument(..., help="Source Probe")):
    """Probe media.

    Args:
        source: source to probe
    """
    pp = ProbeInfo()
    pinfo = pp.probe(source)
    pprint(pinfo)


@main.command()
def probe_with_cache(source: str = typer.Argument(..., help="Source Probe")):
    """Probe media with disk cache.

    Args:
        source: source to probe
    """
    pp = ProbeInfo()
    pinfo = pp.probe_with_cache(source)
    pprint(pinfo)


@main.command()
def probe_info(source: str = typer.Argument(..., help="Source Probe")):
    """Probe media with disk cache.

    Args:
        source: source to probe
    """
    pp = ProbeInfo()
    pinfo = pp.probe_with_cache(source)
    result = pp.info2str(pinfo)
    print(f"result: {result}")


@main.command()
def probe_list(m3u_file: str = typer.Argument(..., help="M3u file")):
    """Probe media.

    Args:
        m3u_list: m3u file
    """
    from .probe_list import ProbeInfoList
    ppl = ProbeInfoList()
    ppl.probe_list(m3u_file)


@main.command()
def m3u_action(action: List[str] = typer.Option(None)):
    """Probe media.

    Args:
        action: actions
    """
    #from .probe_list import ProbeInfoList
    pprint(action)


@main.command()
def run_tasks(
    yaml_command: str = typer.Option(  # noqa: B008
        ..., help="read yaml file", envvar="fhs_xmltv_yaml", prompt=True
    ),
    force_color: bool = typer.Option(  # noqa: B008
        None, "--force-color/--no-color", help="force color in pipelines"
    ),
    include_tag: Optional[List[str]] = typer.Option(None),  # noqa: B008
    exclude_tag: Optional[List[str]] = typer.Option(None),  # noqa: B008
):
    """Run tasks in yaml file.

    Args:
        yaml_command: xmltv file to use
        force_color: force color in pipeline for example
        include_tag: tags from task to include
        exclude_tag: exclude tasks with this tag
    """
    from rich.console import Console
    from .playyaml import play

    config.CONSOLE = Console(force_terminal=force_color)
    play(yaml_command, include_tag, exclude_tag)


@main.command()
def interactive():
    """Run tasks interactive.
    """
    from .playyaml import interactive_run_cmd2

    interactive_run_cmd2()


if __name__ == "__main__":
    sys.exit(main())  # pragma: no cover



"""fhs_iptv_tools - foxhunt software iptv tools, making iptv easier versions """

__version__ = "0.8.3"
__author__ = "Richard de Vos <rdevos72@gmail.com>"
__author_name__ = "Richard de Vos"
__author_email__ = "rdevos72@gmail.com"
__app_name__ = "fhs_iptv_tools"
__project_name__ = "fhs-iptv-tools"
__all__ = []



"""Playyaml file."""


import sys
from . import config
from .playyaml_lib import func_dict_parse, play as play_lib


def play_command_load_m3u_file(task):
    """Play command load_m3u_file.

    Args:
        task: task array

    Returns:
        Good: boolean
    """
    from .probe_list import ProbeInfoList

    task_store = task["store"]
    task_file = task["file"]
    if task_store not in config.STORE:
        config.STORE[task_store] = ProbeInfoList()
    with config.CONSOLE.status(f"Loading m3u file: {task_file}  to store: {task_store}", spinner="dots"):
        config.STORE[task_store].load_m3u_file(task_file)
    return True


def play_command_save_m3u_file(task):
    """Play command save_m3u_file.

    Args:
        task: task array

    Returns:
        Good: boolean
    """
    from .probe_list import ProbeInfoList

    task_store = task["store"]
    task_file = task["file"]
    if task_store not in config.STORE:
        config.STORE[task_store] = ProbeInfoList()
    with config.CONSOLE.status(f"Loading m3u file: {task_file}  to store: {task_store}", spinner="dots"):
        config.STORE[task_store].save_m3u_file(file=task_file, with_tag=task['with_tag'], without_tag=task['without_tag'])
    return True


def play_command_count_channels(task):
    """Play command count channels.

    Args:
        task: task array

    Returns:
        Good: boolean
    """
    from .probe_list import ProbeInfoList

    task_store = task['store']
    if task_store not in config.STORE:
        config.STORE[task_store] = ProbeInfoList()
    count = config.STORE[task_store].count_channels()
    print(f"amount of channels: {count}")
    return True


def play_command_dump_channels(task):
    """Play command dump channels.

    Args:
        task: task array

    Returns:
        Good: boolean
    """
    from .probe_list import ProbeInfoList, LoadType
    from pprint import pprint

    task_store = task['store']
    if task_store not in config.STORE:
        config.STORE[task_store] = ProbeInfoList()
    channels = config.STORE[task_store].filter_lijst(LoadType.ALL)
    pprint(channels)
    return True


def play_command_delete_channels(task):
    """Play command delete channels.

    Args:
        task: task array

    Returns:
        Good: boolean
    """
    from .probe_list import ProbeInfoList, LoadType

    task_store = task['store']
    if task_store not in config.STORE:
        config.STORE[task_store] = ProbeInfoList()
    count = config.STORE[task_store].delete_channels(with_tag=task['with_tag'], without_tag=task['without_tag'])
    print(f"removed {count} channels.")
    return True


def play_command_list_channels(task):
    """Play command list channels.

    Args:
        task: task array

    Returns:
        Good: boolean
    """
    from .probe_list import ProbeInfoList

    task_store = task['store']
    if task_store not in config.STORE:
        config.STORE[task_store] = ProbeInfoList()
    count = config.STORE[task_store].list_channels(with_tag=task['with_tag'], without_tag=task['without_tag'])
    print(f"list {count} channels.")
    return True


def play_command_modify_channels(task):
    """Play command modify channels.

    Args:
        task: task array

    Returns:
        Good: boolean
    """
    from .probe_list import ProbeInfoList

    task_store = task['store']
    if task_store not in config.STORE:
        config.STORE[task_store] = ProbeInfoList()
    count = config.STORE[task_store].modify_channels(
        with_tag=task['with_tag'],
        without_tag=task['without_tag'],
        with_id=task['with_id'],
        set_id=task['set_id'],
        set_name=task['set_name'],
        set_group_title=task['set_group_title'],
        set_logo=task['set_logo'],
    )

    print(f"modified {count} channels.")
    return True


def play_command_select(task):
    """Play command select.

    Args:
        task: task array

    Returns:
        Good: boolean
    """
    from .probe_list import ProbeInfoList

    task_store = task['store']
    if task_store not in config.STORE:
        config.STORE[task_store] = ProbeInfoList()
    count = config.STORE[task_store].select(
        with_tag=task['with_tag'],
        without_tag=task['without_tag'],
        tvg_group_title=task['group_title'],
        tvg_name=task['name'],
        tvg_id=task['id'],
        tvg_source=task['source'],
        set_tag=task['set_tag'],
        clear_tag=task['clear_tag'])
    print(f"selected {count} channels.")
    return True


def play_command_add_channel(task):
    """Play command add channel.

    Args:
        task: task array

    Returns:
        Good: boolean
    """
    from .probe_list import ProbeInfoList

    task_store = task['store']
    if task_store not in config.STORE:
        config.STORE[task_store] = ProbeInfoList()
    config.STORE[task_store].add_channel(tvg_id=task['id'], tvg_name=task['name'], tvg_logo=task['logo_url'], tvg_group_title=task['group_title'], tvg_source=task['source'])
    print(f"add channel: {task['id']} / {task['name']}")
    return True


def play_command_probe_scan(task):
    """Play command probe scan

    Args:
        task: task array

    Returns:
        Good: boolean
    """
    from .probe_list import ProbeInfoList

    task_store = task['store']
    delay = int(task['delay'])
    if task_store not in config.STORE:
        config.STORE[task_store] = ProbeInfoList()
    with config.CONSOLE.status(f"Probe scan channel list from store: {task_store} with delay {delay}", spinner="dots"):
        config.STORE[task_store].probe_scan(delay)
    return True


def play_command_list_groups(task):
    """Play command list groups.

    Args:
        task: task array

    Returns:
        Good: boolean
    """
    from .probe_list import ProbeInfoList

    task_store = task['store']
    if task_store not in config.STORE:
        config.STORE[task_store] = ProbeInfoList()
    groups = config.STORE[task_store].list_groups()
    for i in groups:
        print(i)
    return True


def play_command_group_channels(task):
    """Play command group channels.

    Args:
        task: task array

    Returns:
        Good: boolean
    """
    from .probe_list import ProbeInfoList

    task_store = task['store']
    if task_store not in config.STORE:
        config.STORE[task_store] = ProbeInfoList()
    channels = config.STORE[task_store].group_channels(task['group'])
    for ch in channels:
        print(ch.tvg_name)
    return True


def play_command_filter_channels(task):
    """Play command filter channels.

    Args:
        task: task array

    Returns:
        Good: boolean
    """
    from .probe_list import LoadType, ProbeInfoList

    task_store = task['store']
    task_type = task['type']
    if task_store not in config.STORE:
        config.STORE[task_store] = ProbeInfoList()
    if task_type == 'channels':
        with config.CONSOLE.status(f"Filter m3u list on channels to store: {task_store}", spinner="dots"):
            config.STORE[task_store].filter_lijst(LoadType.CHANNELS)
        return True
    if task_type == 'vod':
        with config.CONSOLE.status(f"Filter m3u list on vod to store: {task_store}", spinner="dots"):
            config.STORE[task_store].filter_lijst(LoadType.VOD)
        return True
    sys.stderr.write(f"ERROR: unknown filter options '{task_type}' and not channels or vod")
    return True


funcdict = {
    "load_m3u": {
        "args": [{"name": "file"}, {"name": "store", "help": "store name", "default": "default"}],
        "func": play_command_load_m3u_file,
        "help": "loading m3u file from disk."
    },
    "count_channels": {
        "args": [{"name": "store", "help": "store name", "default": "default"}],
        "func": play_command_count_channels,
        "help": "count amount of channels in m3u list."
    },
    "dump_channels": {
        "args": [{"name": "store", "help": "store name", "default": "default"}],
        "func": play_command_dump_channels,
        "help": "dump channels using pprint."
    },
    "list_groups": {
        "args": [{"name": "store", "help": "store name", "default": "default"}],
        "func": play_command_list_groups,
        "help": "list groups."
    },
    "group_channels": {
        "args": [{"name": "store", "help": "store name", "default": "default"},{"name": "group"}],
        "func": play_command_group_channels,
        "help": "list channels for a group."
    },
    "probe_scan": {
        "args": [{"name": "store", "help": "store name", "default": "default"},{"name": "delay", "help": "delay between probing channels", "default": "5"}],
        "func": play_command_probe_scan,
        "help": "probe scanning the list of channels."
    },
    "filter_channels": {
        "args": [{"name": 'type', "help": "type to filter, channels or vod"}, {"name": "store", "help": "store name", "default": "default"}],
        "func": play_command_filter_channels,
        "help": "filter channels."
    },
    "add_channel": {
        "args": [
            {"name": "store", "help": "store name", "default": "default"},
            {"name": "id"},
            {"name": "name"},
            {"name": "logo_url", "default": ""},
            {"name": "group_title"},
            {"name": "source"}
        ],
        "func": play_command_add_channel,
        "loop": "channels",
        "help": "add channel."
    },
    "delete_channels": {
        "args": [
            {"name": "store", "help": "store name", "default": "default"},
            {"name": "with_tag", "default": ""},
            {"name": "without_tag", "default": ""},
        ],
        "func": play_command_delete_channels,
        "help": "delete channels."
    },
    "modify_channels": {
        "args": [
            {"name": "store", "help": "store name", "default": "default"},
            {"name": "with_tag", "default": ""},
            {"name": "without_tag", "default": ""},
            {"name": "with_id", "default": ""},
            {"name": "set_id", "default": ""},
            {"name": "set_name", "default": ""},
            {"name": "set_group_title", "default": ""},
            {"name": "set_logo", "default": ""},
        ],
        "func": play_command_modify_channels,
        "loop": "channels",
        "help": "modify channels."
    },
    "list_channels": {
        "args": [
            {"name": "store", "help": "store name", "default": "default"},
            {"name": "with_tag", "default": ""},
            {"name": "without_tag", "default": ""},
        ],
        "func": play_command_list_channels,
        "help": "list channels."
    },
    "save_m3u": {
        "args": [
            {"name": "store", "help": "store name", "default": "default"},
            {"name": "file"},
            {"name": "with_tag", "default": ""},
            {"name": "without_tag", "default": ""},
        ],
        "func": play_command_save_m3u_file,
        "help": "save channels."
    },
    "select": {
        "args": [
            {"name": "store", "help": "store name", "default": "default"},
            {"name": "with_tag", "default": ""},
            {"name": "without_tag", "default": ""},
            {"name": "group_title", "default": ""},
            {"name": "name", "default": ""},
            {"name": "id", "default": ""},
            {"name": "source", "default": ""},
            {"name": "set_tag", "default": ""},
            {"name": "clear_tag", "default": ""}
        ],
        "func": play_command_select,
        "help": "select channels."
    }
}


def play(commandfile, include_tags=None, exclude_tags=None):
    """Play commandfile.

    Args:
        commandfile: yaml file with instructions
        include_tags: list of tags to run
        exclude_tags: list of tags to skip

    Returns:
        None
    """
    play_lib(commandfile, include_tags=include_tags, exclude_tags=exclude_tags, funcdict=func_dict_parse(funcdict))

    return None


def interactive_run_cmd2():
    """Play interactive

    """
    from .interactive_cmd2 import run_cmd2

    run_cmd2(funcdict=func_dict_parse(funcdict, for_type="interactive"))



# probe source

from .probe import ProbeInfo
from .import_m3u import import_m3u_file, return_tvg_group_titles, M3uChannel
from enum import Enum
import time


class LoadType(Enum):
    """LoadTypes from loading m3u channels."""

    ALL = 0
    CHANNELS = 1
    VOD = 2


class ProbeInfoList:
    def __init__(self):
        """Initiate class."""
        self.__probe = ProbeInfo()
        self.__m3u_channels = list()

    def write_error(self, msg):
        """Write error message.

        Args:
            msg: error message
        """
        print(f"ERROR: {msg}")

    def load_m3u_file(self, m3u_file):
        """Load a m3u file.

        Args:
            m3u_file: file to load

        Returns:
            list op m3uchannels
        """
        self.__m3u_channels = import_m3u_file(m3u_file)
        if self.__m3u_channels == None:
            self.write_error(f"can't load m3u_file: {m3u_file}")
        return self.__m3u_channels

    def filter_lijst(self, filter_type):
        """Filter the list based on filter_type.

        Args:
            filter_type: type of LoadType

        Returns:
            filtered list of type.
        """
        include_vod = None
        if filter_type == LoadType.ALL:
            return self.__m3u_channels

        if filter_type == LoadType.CHANNELS:
            include_vod = False

        if filter_type == LoadType.VOD:
            include_vod = True

        if include_vod is None:
            self.write_error(f"unknown type: {filter_type=}")
            return None

        result = list(filter(lambda d: d.vod is include_vod, self.__m3u_channels))
        self.__m3u_channels = result
        return self.__m3u_channels

    def probe_scan(self, delay=5, status_output=None):
        """Probe a m3u file.

        Args: 
            delay: delay between channels.
            status_output: function to output status

        Returns:
            Status: None
        """
        max_len = 10
        for ch in self.__m3u_channels:
            if len(ch.tvg_name) > max_len:
                max_len = len(ch.tvg_name)
        max_len += 3
        for ch in self.__m3u_channels:
            if status_output is not None:
                status_output(f"starting scanning: {ch.tvg_name}")
            if type(ch.tvg_sources) == list:
                for source in ch.tvg_sources:
                    result = self.__probe.probe_with_cache(source)
            else:
                result = self.__probe.probe_with_cache(ch.tvg_sources)
            if result is None:
                if status_output is not None:
                    status_output(f"no result for: {ch.tvg_name}")
                continue

            if status_output is not None:
                status_output(f"ready scanning: {ch.tvg_name}")
            ch.fhs_info = self.__probe.info2str(result)
            print(f"{ch.tvg_name:{max_len}}:\t{ch.fhs_info}")
            if result.get('fhs_source', 'unknown') != 'cache':
                time.sleep(delay)

        return None

    def count_channels(self):
        """Count the channels.

        Returns:
            count of channels
        """
        return len(self.__m3u_channels)

    def list_groups(self, vod_only=False):
        """List the groups.

        Args:
            vod_only: show vod only

        Returns:
            list of groups
        """
        groups = return_tvg_group_titles(self.__m3u_channels, vod_only)
        return groups

    def group_channels(self, group):
        """List the channels in a group.

        Args:
            group: show vod only

        Returns:
            list of channels
        """
        channels = []
        for ch in self.__m3u_channels:
            if ch.tvg_group_title == group:
                channels.append(ch)
        return channels

    def add_channel(self, *, tvg_id, tvg_name, tvg_logo, tvg_group_title, tvg_source):
        """List the channels in a group.

        Args:
            group: show vod only

        Returns:
            list of channels
        """
        self.__m3u_channels.append(M3uChannel(tvg_id=tvg_id, tvg_name=tvg_name.strip(), tvg_logo=tvg_logo, tvg_group_title=tvg_group_title))
        self.__m3u_channels[-1].tvg_sources.append(tvg_source)

    def select(self, *, with_tag="", without_tag="", tvg_group_title="", tvg_name="", tvg_id="", tvg_source="", set_tag="", clear_tag=""):
        """Select channel.

        If multiple filters are used than it is a 'AND'

        Args:
            with_tag: select on tag set
            without_tag: select on tag not set
            tvg_id: select based on tvg_id
            tvg_group_title: select in group_title
            tvg_name: select in tvg_name
            tvg_source: select on source url
            set_tag: set tag
            clear_tag: remove tag

        Returns:
            count, amount of
        """
        from .utils import check_search_filter_in_string

        if set_tag == "" and clear_tag == "":
            self.write_error("ERROR: select needs to set a tag or remove a tag.")
            return -1

        count = 0
        for ch in self.__m3u_channels:
            # check if tag is precent
            if with_tag != "" and with_tag not in ch.fhs_tags:
                continue

            # check if tag is not precent
            if without_tag != "" and without_tag in ch.fhs_tags:
                continue

            if tvg_id != "" and check_search_filter_in_string(ch.tvg_id, tvg_id) is False:
                continue

            if tvg_group_title != "" and check_search_filter_in_string(ch.tvg_group_title, tvg_group_title) is False:
                continue

            if tvg_name != "" and check_search_filter_in_string(ch.tvg_name, tvg_name) is False:
                continue

            if tvg_source != "":
                self.write_error(f"ERROR: tvg_source not implemented: {tvg_source}.")
            count += 1
            if set_tag != "":
                ch.fhs_tags.add(set_tag)
            if clear_tag != "":
                try:
                    ch.fhs_tags.remove(set_tag)
                except KeyError:
                    pass
            print(f"{count}: {ch.tvg_name}     /   {ch.tvg_group_title}")
        return count

    def delete_channels(self, *, with_tag="", without_tag=""):
        """Delete channels.

        Delete channels

        Args:
            with_tag: select on tag set
            without_tag: select on tag not set

        Returns:
            count, amount of
        """

        if with_tag == "" and without_tag == "":
            self.write_error("delete_channel needs with_tag or without_tag set.")
            return -1

        count = 0
        for p in range(len(self.__m3u_channels), 0, -1):
            if with_tag != "" and with_tag in self.__m3u_channels[p - 1].fhs_tags:
                del self.__m3u_channels[p - 1]
                count += 1
                continue
            if without_tag != "" and without_tag not in self.__m3u_channels[p - 1].fhs_tags:
                del self.__m3u_channels[p - 1]
                count += 1
                continue
        return count

    def get_channels(self, *, with_tag="", without_tag="", with_id=""):
        """Get channels.

        Get channels

        Args:
            with_tag: select on tag set
            without_tag: select on tag not set

        Returns:
            yield of channels
        """

        for ch in self.__m3u_channels:
            if with_id != "" and ch.tvg_id != with_id:
                continue
            if with_tag != "" and with_tag not in ch.fhs_tags:
                continue
            if without_tag != "" and without_tag in ch.fhs_tags:
                continue
            yield ch

    def display_channel(self, channel, *, extra=None):
        """Display channel.

        Args:
            channel: m3uchannel to display
            extra: what extra info to show, future use
        """
        tags = ", ".join(channel.fhs_tags)
        if tags != "":
            display_tags = f" tags: {tags}"
        else:
            display_tags = ""
        return f"<{channel.tvg_id}> {channel.tvg_name}   /   {channel.tvg_group_title}{display_tags}"

    def list_channels(self, *, with_tag="", without_tag=""):
        """List channels.

        List channels

        Args:
            with_tag: select on tag set
            without_tag: select on tag not set

        Returns:
            yield of channels
        """

        count = 0
        for ch in self.get_channels(with_tag=with_tag, without_tag=without_tag):
            count += 1
            print(f"{count}: {self.display_channel(ch)}")
        return count

    def save_m3u_file(self, *, file, with_tag="", without_tag=""):
        """List channels.

        List channels

        Args:
            file: filename to save
            with_tag: select on tag set
            without_tag: select on tag not set
        """
        from .import_m3u import export_m3u_file

        channels = self.get_channels(with_tag=with_tag, without_tag=without_tag)
        if export_m3u_file(file, channels) is False:
            print(f"save failed to file: {file}.")
        print(f"channels saved to m3u file {file}")

    def modify_channels(self, *, with_tag="", without_tag="", with_id="", set_id="", set_name="", set_group_title="", set_logo=""):
        """Modify channel.

        Args:
            with_tag: select on tag set
            without_tag: select on tag not set
            with_id: change only on id
            set_id: set new id
            set_name: set_name
            set_group_title: set group title
            set_logo: set logo

        Returns:
            count: channels changed.
        """
        channels = self.get_channels(with_tag=with_tag, without_tag=without_tag, with_id=with_id)
        if set_id != "" or set_name != "" or set_logo != "":
            # this modifications we do only on one channel at a time.
            channels = list(channels)
            if len(channels) != 1:
                self.write_error(f"ERROR: you can only change this type one channel on a time and not for {len(channels)}")
                return -1
        count = 0
        for ch in channels:
            count += 1
            if set_id != "":
                ch.tvg_id = set_id
            if set_name != "":
                ch.tvg_name = set_name
            if set_group_title != "":
                ch.tvg_group_title = set_group_title
            if set_logo != "":
                ch.tvg_logo = set_logo
        return count





import subprocess
import json
import hashlib
import os
from pathlib import Path
from .directories import get_probe_dir
from pprint import pprint


class ProbeInfo:
    def __init__(self, cmdpath="/usr/bin/ffprobe", seed="Pn2OOHXBFjnSqB.Yt", probedir=None):
        """Initiate class.

        Args:
            cmdpath: ffprobe command part
            seed: seed for sha1 function
            probedir: directory to cache probes info
        """
        self._cmdpath = cmdpath
        self._seed = seed
        self._probedir = probedir

    def process_media_info(self, media_info):
        """Cleanup and process media info.

        Args:
            media_info: probe info

        Returns:
            cleaned media_info
        """
        try:
            del media_info['format']['filename']
        except KeyError:
            pass
        return media_info

    def probe(self, source):
        """Probe a source.

        Args:
            source: source to probe

        Returns:
            mediainfo
        """
        cmd = [
            self._cmdpath,
            "-loglevel",
            "quiet",
            "-print_format",
            "json",
            "-show_format",
            "-show_streams",
            "-show_error",
            "-show_packets",
            "-count_frames",
            "-read_intervals",
            "%+10",
            "-i",
            source,
        ]  # maybe add -read_intervals  %+10
        try:
            outputBytes = subprocess.check_output(cmd)
        except subprocess.CalledProcessError as e:
            self._last_error = str(e)
            return None

        outputText = outputBytes.decode("utf-8")
        try:
            ffprobe = json.loads(outputText)
        except json.JSONDecodeError as e:
            self._last_error = str(e)
            return None
        return self.process_media_info(ffprobe)

    def get_cache_path(self, source):
        """Get cache path.

        Args:
            source: source to create path from

        Returns:
            path
        """
        if self._probedir is None:
            self._probedir = get_probe_dir()

        hash_string = f"{source}_{self._seed}"
        hash_object = hashlib.sha1(hash_string.encode())
        return os.path.join(self._probedir, f"{hash_object.hexdigest()}.json")

    def load_cache_source(self, source):
        """Get cached probe

        Args:
            source: source to get cache from

        Returns:
            mediainfo
        """
        probe_file = self.get_cache_path(source)
        try:
            with Path(probe_file).open(encoding="UTF-8") as source:
                media_info = json.load(source)
        except FileNotFoundError:
            media_info = None
        return media_info

    def save_cache_source(self, source, media_info):
        """Save probe to cache.

        Args:
            source: source to get cache from
            media_info: probe_info to cache

        Returns:
            Success
        """
        probe_file = self.get_cache_path(source)
        with Path(probe_file).open("w", encoding="UTF-8") as target:
            json.dump(media_info, target)
        return True

    def probe_with_cache(self, source):
        """Probe a source cache to or from disk.

        Args:
            source: source to probe

        Returns:
            mediainfo
        """
        media_info = self.load_cache_source(source)
        if media_info is not None:
            media_info['fhs_source'] = 'cache'
            return media_info

        media_info = self.probe(source)
        if media_info is not None:
            self.save_cache_source(source, media_info)
            media_info['fhs_source'] = 'ffprobe'
        return media_info
 
    def calculate_bitrate(self, media_info, stream):
        """Calculate bitrate.

        Args:
            media_info: ffprobe info
            stream: stream id to check

        Returns:
            calculated bitrate
        """
        total_size = 0
        total_time = float()
        for packet in media_info['packets']:
            if packet['stream_index'] != stream:
                continue
            total_size += int(packet['size'])
            total_time += float(packet.get('duration_time', 0))
        try:
            bitrate = int(((total_size * 8) / total_time))
        except ZeroDivisionError:
            return "unknown"
        return f"{bitrate} calculated"

    def swap_video_and_audio_in_list(self, result):
        """Reverse list is list consist of audio / video and not video / auto.

        Args:
            result: list of results

        Returns:
            list in order video / audio
        """
        if len(result) != 2:
            return result  # more or less than 2 entries
        if 'audio' in result[0] and 'video' in result[1]:
            result.reverse()
        return result

    def info2str(self, media_info):
        """Convert media info to info str.

        Args:
            media_info: ffprobe info

        Returns:
            media_str
        """
        result = []
        for idx, p in enumerate(media_info['streams']):
            codec_type = p.get('codec_type', 'unknown')
            if 'bit_rate' in p:
                bit_rate = p['bit_rate']
            else:
                bit_rate = self.calculate_bitrate(media_info, idx)
            if codec_type == 'video':
                result.append(f"video: {p.get('codec_name','codec unknown')} bit_rate: {bit_rate} / width: {p.get('width', 'unknown')} / height: {p.get('height', 'unknown')}")  # noqa: E501
            if codec_type == 'audio':
                result.append(f"audio: {p.get('codec_name','codec unknown')} bit_rate: {bit_rate}")  # noqa: E501
        result = self.swap_video_and_audio_in_list(result)
        return ", ".join(result)



"""Subgroup m3u"""

def subgroup_m3uchannels(m3uchannels, subgroup_name):
    """Subgroup m3u stream."""

    result = filter(lambda d: d.tvg_group_title == subgroup_name, m3uchannels)
    return result


def subgroup_vod_only(m3uchannels, include_vod=True):
    result = filter(lambda d: d.vod is include_vod, m3uchannels)
    return result

